#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
from PySide2.QtWidgets import QApplication, QMainWindow, QMenuBar, QAction, QWidget, QVBoxLayout, QFileDialog, QDialog
from PySide2.QtCore import qtTrId
from mkedit.core.EditCenter import EditCenter
from mkedit.core.Dialog import Dialog as MKDialog
import os

# print(os.environ[
#     'QTWEBENGINEPROCESS_PATH'])
# os.environ[
#     'QTWEBENGINEPROCESS_PATH'] = '/Users/angels/github/PySide2MkEdit/dist/main.app/Contents/MacOS/QtWebEngineProcess.app/Contents/MacOS/QtWebEngineProcess'


print(os.environ)
class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        QMainWindow.__init__(self, parent)
        self.init()
        self.initSize()
        self.initUI()
        self.initData()
        self.updateMenuBar()

    def init(self):
        self.editCenter = EditCenter(self)

    def initSize(self):
        desktop = QApplication.desktop()
        self.screenWidth = desktop.width() * 0.8
        self.screenHeight = desktop.height() * 1
        print("screen width is %d height is %d" % (self.screenWidth, self.screenHeight))
        self.resize(self.screenWidth, self.screenHeight)

    def initUI(self):
        self.mainWidget = QWidget()
        self.mainLayout = QVBoxLayout()
        self.mainWidget.setLayout(self.mainLayout)
        self.mainLayout.addWidget(self.editCenter)
        self.setCentralWidget(self.mainWidget)

    def updateMenuBar(self):
        menuBar = QMenuBar()
        winmenu = menuBar.addMenu(qtTrId("File"))
        selectProjectDirAction = QAction(qtTrId("选择工作目录"), self)
        selectProjectDirAction.triggered.connect(self.openFileDir)
        winmenu.addAction(selectProjectDirAction)

        createDirAction = QAction(qtTrId("创建目录"), self)
        createDirAction.triggered.connect(self.createDir)
        winmenu.addAction(createDirAction)

        createFileAction = QAction(qtTrId("创建文件"), self)
        createFileAction.triggered.connect(self.createFile)
        winmenu.addAction(createFileAction)
        self.setMenuBar(menuBar)

    def initData(self):
        self.editCenter.getMenusBar.updateWorkDir("")
        self.editCenter.getMenusBar.setDefaultValue("")

    def openFileDir(self):
        print("openFileDir")
        fileDialog = QFileDialog(self)
        fileDialog.setFileMode(QFileDialog.Directory)
        ret = fileDialog.exec_()
        if ret == QDialog.Accepted:
            fileName = fileDialog.selectedFiles()
            self.editCenter.getMenusBar.updateWorkDir(fileName[0])

    def createDir(self):
        print("createDir")
        text = MKDialog().inputDirNameDialog()
        if text:
            print(text)
            self.editCenter.getMenusBar.createDir(text)

    def createFile(self):
        print("createFile")
        text = MKDialog().inputFileNameDialog()
        if text:
            print(text)
            self.editCenter.getMenusBar.createFile(text, self.editCenter.getMenusBar.getDefaultValue)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    mainWin = MainWindow()
    mainWin.show()
    sys.exit(app.exec_())


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide2.QtWidgets import QWidget, QFileSystemModel, QTreeView, QAction, QMenu, QVBoxLayout, QAbstractItemView
from PySide2 import QtCore
from PySide2.QtCore import QMargins, QFileInfo
from PySide2.QtGui import QMouseEvent
from .FileUtils import createFile, createDir, deleteDir, deleteFile
from .DataClass import FileInfo
import uuid
from rxbus.core import RxBus
from .Dialog import Dialog as MKDialog
import json
import os
from .FileUtils import createFile


class MemusBar(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.lastSettingFile = "lastSettingFile"
        self.init()
        self._initUI()
        self.defaultForCreateMD = ["metaData-id:%s" % uuid.uuid1(), "metaData-title:"]
        self.initEvent()

    def __del__(self):
        RxBus.instance.unRegister(self)

    def init(self):
        self.workDir = ""

    def createDir(self, value):
        createDir(self.workDir, value)

    def createFile(self, value, default: str = ""):
        createFile(self.workDir, value, self.defaultForCreateMD)

    def initEvent(self):
        pass

    def getSettingFile(self):
        if not os.path.exists("lastSettingFile.json"):
            with open('lastSettingFile.json', mode='w')  as jsonFile:
                pass
        with open('lastSettingFile.json', mode='r') as jsonFile:
            try:
                data = json.load(jsonFile)
                return data
            except Exception as e:
                print(e)
                return {}

    def saveSettingFile(self, setting):
        with open('lastSettingFile.json', mode='w') as jsonFile:
            json.dump(setting, jsonFile)

    def getLastWorkDirRecord(self):
        setting: dict = self.getSettingFile()
        value = setting.get("lastWorkDirRecord")
        if value:
            return value
        else:
            return "/"

    def saveWorkDirRecord(self, value):
        setting = self.getSettingFile()
        setting["lastWorkDirRecord"] = value
        self.saveSettingFile(setting)

    def updateWorkDir(self, value: str = ""):
        if not value or value == "":
            # 去最后一次的目录信息
            self.workDir = self.getLastWorkDirRecord()
        else:
            self.workDir = value
            self.saveWorkDirRecord(value)
        self.treeModel.setRootPath(self.workDir)
        self.treeView.setRootIndex(self.treeModel.index(self.workDir))
        self.treeModel.setNameFilters(['*.md'])

    def _initUI(self):
        self.mainLayout = QVBoxLayout(self)

        self.mainLayout.setContentsMargins(QMargins(0, 0, 0, 0))

        self.treeModel = QFileSystemModel(self)
        self.treeModel.setReadOnly(False)
        self.treeView = QTreeView(self)
        self.treeView.setDragDropMode(QAbstractItemView.InternalMove)
        self.treeView.setDragEnabled(True)
        self.treeView.setAcceptDrops(True)
        self.treeView.setDropIndicatorShown(True)
        self.setMinimumHeight(100)
        self.treeView.setModel(self.treeModel)
        self.treeView.setSortingEnabled(True)
        self.treeView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.treeView.customContextMenuRequested.connect(self.showContextMenu)
        self.treeView.mouseDoubleClickEvent = self.mouseDoubleClickEvent
        self.treeView.setColumnHidden(1, True)
        self.treeView.setColumnHidden(2, True)
        self.treeView.setColumnHidden(3, True)
        self.mainLayout.addWidget(self.treeView, 1)
        self.setLayout(self.mainLayout)

    def mouseDoubleClickEvent(self, event: QMouseEvent):
        if event.button() == QtCore.Qt.LeftButton:
            # 是否按下左键
            index = self.treeView.indexAt(event.pos())
            if index.isValid():
                fileInfo = self.treeModel.fileInfo(index)
                if fileInfo.isFile():
                    fileInfo = FileInfo(fileInfo.fileName(), fileInfo.filePath())
                    self.openFileCallBack(fileInfo)
        else:
            event.accept()

    def showContextMenu(self, pos):
        index = self.treeView.indexAt(pos)
        if index.isValid():
            menu = QMenu(self)
            action = QAction(menu)
            action.setObjectName("create_dir")
            action.setProperty("pos", pos)
            action.setText("创建目录")
            action.triggered.connect(self.actionHandler)
            menu.addAction(action)

            action = QAction(menu)
            action.setObjectName("delete_dir")
            action.setProperty("pos", pos)
            action.setText("删除目录")
            action.triggered.connect(self.actionHandler)
            menu.addAction(action)

            action = QAction(menu)
            action.setObjectName("create_file")
            action.setProperty("pos", pos)
            action.setText("创建文件")
            action.triggered.connect(self.actionHandler)
            menu.addAction(action)

            action = QAction(menu)
            action.setObjectName("delete_file")
            action.setProperty("pos", pos)
            action.setText("删除文件")
            menu.addAction(action)
            action.triggered.connect(self.actionHandler)
            menu.exec_(self.treeView.mapToGlobal(pos))

    def actionHandler(self):
        print("actionHandler")
        type = self.sender().objectName()
        pos = self.sender().property("pos")
        print("pos:%s" % pos)
        fileInfo = self.treeModel.fileInfo(self.treeView.currentIndex())
        print("fileInfo %s" % fileInfo)
        print(fileInfo.fileName())
        print(fileInfo.filePath())
        print(fileInfo.path())
        print(fileInfo.isRoot())
        path = ""
        file = fileInfo.filePath()
        print("filePath: %s" % fileInfo.filePath())
        print("path: %s" % fileInfo.path())
        print("fileName: %s" % fileInfo.fileName())

        path = fileInfo.path()

        # if path == AppConfig.articleRootPath:
        #     return
        if type == "create_dir":
            print("创建目录")
            text = MKDialog().inputDirNameDialog()
            if text:
                print(text)
                createDir(self.getDir(fileInfo), text)
            if fileInfo.isDir():
                print("是目录")
        elif type == "delete_dir":
            print("删除目录")
            deleteDir(fileInfo.filePath())
        elif type == "create_file":
            print("创建文件")
            text = MKDialog().inputFileNameDialog()
            if text:
                print(text)
                createFile(self.getDir(fileInfo), text)
        elif type == "delete_file":
            print("删除文件")
            deleteFile(file)

    def setDefaultValue(self, value: str = ""):
        self.defaultForCreateFile = value

    @property
    def getDefaultValue(self):
        return self.defaultForCreateMD

    def getDir(self, fileInfo: QFileInfo):
        if fileInfo.isFile():
            return fileInfo.path()
        else:
            return fileInfo.filePath()

    def openFileCallBack(self, value):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .MkEdit import MkEdit
from .Dialog import Dialog
from .DataClass import FileInfo, WorkdirInfo, CreateDirInfo, CreateFileInfo
from .EditCenter import EditCenter



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import List


class FileInfo:
    def __init__(self, fileName: str = "", filePath: str = "", data: List = None):
        self.fileName = fileName
        self.filePath = filePath
        self.data = data


class WorkdirInfo:
    def __init__(self, dirPath: str = ""):
        self.dirPath = dirPath


class CreateDirInfo:
    def __init__(self, dirPath: str = ""):
        self.dirPath = dirPath


class CreateFileInfo:
    def __init__(self, dirPath: str = ""):
        self.dirPath = dirPath


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWidgets import QWidget, QVBoxLayout, QTabWidget, QHBoxLayout
from PySide2.QtCore import QMargins
from .DataClass import FileInfo
from .MkEdit import MkEdit
from .MenusBar import MemusBar


class EditCenter(QWidget):
    def __init__(self, parent=None):
        print("EditCenter __init__")

        QWidget.__init__(self, parent)
        self._initUI()
        self._initData()
        self._initEvent()

    def __del__(self):
        print("EditCenter __del__")

    def _initEvent(self):
        self.getMenusBar.openFileCallBack = self.openFileCallBack

    def openFileCallBack(self, value):
        self.openFileInfo(value)

    def openFileInfo(self, value: FileInfo):
        has = False
        for index in range(0, self.mdiare.count()):
            tab = self.mdiare.widget(index)
            if tab.property("title") == value.filePath:
                self.mdiare.setCurrentIndex(index)
                has = True
                break
        if not has:
            edit = MkEdit(self)
            edit.loadData(value.filePath)
            edit.setProperty("title", value.filePath)
            self.mdiare.addTab(edit, value.fileName)
            self.mdiare.setTabToolTip(self.mdiare.count() - 1, value.filePath)
            self.mdiare.setCurrentIndex(self.mdiare.count() - 1)

    def tabCloseRequested(self, index):
        _currentTab: MkEdit = self.mdiare.widget(index)
        if not _currentTab.interceptClose():
            self.mdiare.removeTab(index)

    def tabBarClicked(self, index):
        print("tabBarClicked index : %s" % index)

    def tabBarDoubleClicked(self, index):
        print("tabBarDoubleClicked index : %s" % index)

    def currentChanged(self, index):
        print("currentChanged index : %s" % index)

    def destroy(self, destroyWindow: bool = ..., destroySubWindows: bool = ...):
        print("destroy")
        QWidget(EditCenter, self).destroy(destroyWindow, destroySubWindows)

    def create(self, arg__1: int = ..., initializeWindow: bool = ..., destroyOldWindow: bool = ...):
        print("create")
        QWidget(EditCenter, self).create(arg__1, initializeWindow, destroyOldWindow)

    def _initUI(self):
        self.mainLayout = QHBoxLayout(self)
        self.mainLayout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.menuBar = MemusBar(self)
        self.mainLayout.addWidget(self.menuBar)
        self.rightLayout = QVBoxLayout(self)
        self.rightLayout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.mdiare = QTabWidget(self)
        self.mdiare.setDocumentMode(True)
        # 注册关闭按钮信号
        self.mdiare.tabCloseRequested.connect(self.tabCloseRequested)
        self.mdiare.tabBarClicked.connect(self.tabBarClicked)
        self.mdiare.tabBarDoubleClicked.connect(self.tabBarDoubleClicked)
        self.mdiare.currentChanged.connect(self.currentChanged)
        self.mdiare.setMovable(True)
        self.mdiare.setTabsClosable(True)
        self.rightLayout.addWidget(self.mdiare)
        self.mainLayout.addLayout(self.rightLayout, 1)
        self.setLayout(self.mainLayout)

    def _initData(self):
        pass
        # edit = Edit(self)
        # self.mdiare.addTab(edit, "xxxx")
        # for i in range(10):

    @property
    def getMenusBar(self):
        return self.menuBar



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PySide2.QtWidgets import QVBoxLayout, QWidget, QHBoxLayout, QTextBrowser, QPlainTextEdit, QSizePolicy, QMessageBox, QShortcut
from mistune import Markdown, Renderer
from PySide2.QtGui import QFont, QFontMetrics, QKeySequence, QKeyEvent
from PySide2.QtCore import QMargins
from os import path
from rx import operators as ops, scheduler
from .FileUtils import updateFile


class RightWebView(QTextBrowser):
    def __init__(self, parent=None):
        super(RightWebView, self).__init__(parent)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.__html = ''

    @property
    def html(self):
        return self.__html

    @html.setter
    def html(self, html):
        self.__html = html
        self.setHtml(html)

    def notifyScroll(self, dx, dy):
        print("notifyScroll dx:%s dy:%s" % (dx, dy))
        self.scroll(dx, dy)


class LeftEdit(QPlainTextEdit):
    def __init__(self, parent=None):
        QPlainTextEdit.__init__(self, parent)
        self.init()
        self.haseSave = True

    def init(self):
        self.scrollCallBack = None

    def scroll(self, dx: int, dy: int):
        print("scroll dx:%s dy:%s" % (dx, dy))
        super(LeftEdit, self).scroll(dx, dy)

    def scrollContentsBy(self, dx: int, dy: int):
        print("scrollContentsBy dx:%s dy:%s" % (dx, dy))
        super(LeftEdit, self).scrollContentsBy(dx, dy)
        if self.scrollCallBack:
            self.scrollCallBack(dx, dy)

    def setScrollCallBack(self, callback):
        self.scrollCallBack = callback

    def keyPressEvent(self, e: QKeyEvent):
        print(e.key())
        print(e.text())
        if e.key() == 16777217:
            print("tab 被按下")
            self.handleTest(e)
        else:
            super(LeftEdit, self).keyPressEvent(e)

    def handleTest(self, e: QKeyEvent):
        tab = "\t"

        cursor = self.textCursor()

        if cursor.hasSelection():
            start = cursor.selectionStart()
            end = cursor.selectionEnd()
            # 设置游标位置到最后
            cursor.setPosition(end)

            # 移动到选择的行的最后位置
            cursor.movePosition(cursor.StartOfLine)
            # 获取选择的段落的最终位置
            end = cursor.position()
            # cursor.movePosition(cursor.StartOfBlock)
            # endNumber = cursor.blockNumber()

            # 游标恢复到选择的开始位置
            cursor.setPosition(start)
            # cursor.movePosition(cursor.StartOfBlock)
            # startNumber = cursor.blockNumber()
            # print("startNumber = %s and endNumber = %s" % (startNumber, endNumber))
            # self.insertTab(startNumber, endNumber)
            # 移动到开始的问题的行头
            cursor.movePosition(cursor.StartOfLine)
            # 获取开始位置
            start = cursor.position()

            while start < end:
                cursor.movePosition(cursor.StartOfLine)

                _left = cursor.position()
                cursor.movePosition(cursor.EndOfLine)
                _right = cursor.position()

                cursor.movePosition(cursor.StartOfLine)
                cursor.insertText(tab)
                cursor.movePosition(cursor.NextBlock)
                cursor.movePosition(cursor.StartOfLine)
                lineLenght = _right - _left
                start = start + lineLenght
                # cursor.movePosition(cursor.StartOfLine)
        else:
            super(LeftEdit, self).keyPressEvent(e)

    def insertTab(self, satrt, end):
        self.document()
        txt = self.toPlainText()
        lines = txt.split("\n")
        for index, value in enumerate(lines):
            if index >= satrt and index <= end:
                lines[index] = "%s%s\n" % ("\t", value)
            else:
                lines[index] = "%s\n" % value

        self.setPlainText("".join(lines))


class MkEdit(QWidget):
    def __init__(self, parent=None):
        print("EditWidget __init__")
        QWidget.__init__(self, parent)
        self.init()
        self.initUI()
        self.initEvent()

    def interceptClose(self):
        if not self.haseSave:
            msgBox = QMessageBox()
            msgBox.setText("%s-文档已被修改" % self.fileName)
            msgBox.setInformativeText("是否保存修改后的文档")
            msgBox.setStandardButtons(QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)
            msgBox.setDefaultButton(QMessageBox.Save)
            ret = msgBox.exec_()
            print("ret:%s" % ret)
            if ret == QMessageBox.Save:
                print("save")
                self.save()
                self.haseSave = True
            elif ret == QMessageBox.Cancel:
                print("cancel")
                self.haseSave = False
            elif ret == QMessageBox.Discard:
                return False

        return not self.haseSave

    def save(self):
        if not self.haseSave:
            print("satrt save data")
            updateFile(self.filePath, self.leftEdit.toPlainText()).pipe(
                ops.subscribe_on(scheduler.ThreadPoolScheduler())
            ).subscribe(on_completed=lambda: self.handlerSaveResult())

    def handlerSaveResult(self):
        self.haseSave = True

    def loadData(self, filePath):
        self.filePath = filePath
        self.fileName = path.basename(self.filePath)
        result = list()
        with open(self.filePath, mode='r') as f:
            # for line in f.readlines():
            #     result.append(line)
            self.leftEdit.setPlainText("".join(f.readlines()))

    def __del__(self):
        print("EditWidget __del__")

    def setPlainText(self, text: str = ""):
        self.leftEdit.setPlainText(text)

    def init(self):
        renderer = Renderer(escape=True, hard_wrap=True)

        self.markdown = Markdown(renderer=renderer)

    def initUI(self):
        self.mainLayout = QVBoxLayout(self)
        self.mainLayout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.contentLayout = QHBoxLayout(self)
        self.contentLayout.setSpacing(0)
        self.leftEdit = LeftEdit(self)
        self.initLeftEdit()
        self.rightWebView = RightWebView(self)
        self.contentLayout.addWidget(self.leftEdit, 1)
        self.contentLayout.addWidget(self.rightWebView, 1)
        self.mainLayout.addLayout(self.contentLayout)
        self.setLayout(self.mainLayout)

    def initLeftEdit(self):
        shortcut = QShortcut(QKeySequence("Ctrl+S"), self.leftEdit)
        shortcut.activated.connect(self.save)
        font = QFont("Menlo", 14)
        self.leftEdit.setFont(font)
        self.leftEdit.setTabStopWidth(4 * QFontMetrics(font).width(" "))

    def initEvent(self):
        self.leftEdit.textChanged.connect(self.leftEditTextChange)

    def leftEditTextChange(self):
        self.haseSave = False
        self.rightWebView.setHtml(self.markdown.parse(self.leftEdit.toPlainText()))
